#ifndef __MEX_HELPER_H__

#include <ctl.h>
#include <mex.h>

void initEnv();
bool mexTypeCheck (const mxArray *mval, const char *wantType, int i=1, int j=1);

template<class T> class matrix
{
	ctl::uint8 n;
	std::vector<T> data;

	public: 
	CTL_Template(matrix, (T), 1, tupel, (n, data), 2);

	const int N () const
	{
		return n;
	}

	const int M () const
	{
		return data.size()/n;
	}

	const int size () const
	{
		return data.size();
	}

	matrix () : n(0)
	{
	}

	matrix (int n, int m)
	{
		resize(n, m);
	}

	void resize (int n, int m)
	{
		this->n = n;
		data.resize(n*m);
	}

	const T &operator[] (int i) const
	{
		return data[i];
	}

	T &operator[] (int i)
	{
		return data[i];
	}
};

// Compile-time assertion for unsupported conversions
template<bool> struct Assert;
struct Assert<true> {};

template<class T> inline bool mexConvert (const T &cval, mxArray *&mval)
{
	Assert<false> Conversion_not_supported;
	return false;
}

template<> inline bool mexConvert (const std::vector<double> &vec, mxArray *&mval)
{
	mval = mxCreateDoubleMatrix(vec.size(), 1, mxREAL);
	double *values = mxGetPr(mval);
	for (int i=0;i<vec.size();i++)
		values[i] = vec[i];
	return true;
}

template<> inline bool mexConvert (const matrix<double> &mat, mxArray *&mval)
{
	mval = mxCreateDoubleMatrix(mat.N(), mat.M(), mxREAL);
	double *values = mxGetPr(mval);
	for (int i=0;i<mat.size();i++)
			values[i] = mat[i];
	return true;
}

template<> inline bool mexConvert (const std::vector<std::vector<double> > &vec,
	mxArray *&mval)
{
	return false;
}

template<> inline bool mexConvert (const double &d, mxArray *&mval)
{
	mval = mxCreateDoubleScalar(d);
	return true;
}

template<> inline bool mexConvert (const int &i, mxArray *&mval)
{
	mval = mxCreateDoubleScalar(i);
	return true;
}

template<> inline bool mexConvert (const std::string &str, mxArray *&mval)
{
	return false;
}

template<class T> inline bool mexConvert (const mxArray *mval, T &cval)
{
	Assert<false> Conversion_not_supported;
	return false;
}

template<> inline bool mexConvert (const mxArray *mval, std::vector<double> &vec)
{
	if (!mexTypeCheck(mval, "double"))
		return false;
	int len = mxGetN(mval);
	vec.resize(len);
	double *values = mxGetPr(mval);
	for (int i=0;i<len;i++)
		vec[i] = values[i];
	return true;
}

template<> inline bool mexConvert (const mxArray *mval, matrix<double> &mat)
{
	if (!mexTypeCheck(mval, "double"))		
		return false;
	int N = mxGetN(mval);
	int M = mxGetM(mval);
	mat.resize(N, M);
	double *values = mxGetPr(mval);
	for (int i=0;i<N*M;i++)
		mat[i] = values[i];
	return true;
}

template<> inline bool mexConvert (const mxArray *mval, std::vector<std::vector<double> > &vec)
{
	if (!mexTypeCheck(mval, "double"))
		return false;
	int N = mxGetN(mval);
	int M = mxGetM(mval);
	vec.resize(N);
	double *values = mxGetPr(mval);
	for (int i=0;i<N;i++)
	{
		vec.resize(M);
		for (int j=0;j<M;j++)
			vec[i][j] = values[i*M+j];
	}
	return true;
}

template<> bool inline mexConvert (const mxArray *mval, double &cval)
{
	if (!mexTypeCheck(mval, "double"))
		return false;
	cval = mxGetScalar(mval);
	return true;
}

template<> bool inline mexConvert (const mxArray *mval, int &cval)
{
	if (!mexTypeCheck(mval, "double"))
		return false;
	cval = (int)mxGetScalar(mval);
	return true;
}

template<> bool inline mexConvert (const mxArray *mval, std::string &cval)
{
	return false;
}

#endif
