package CTL;

import ReflWrap.*;

import java.util.*;

/** 
 * This class holds mappings from the in-stream type identifier strings to
 * actual Java classes and vice versa. 
 */
public class AnyObjIdentMap
{
	/** Mapping from type identifier to class */
	private HashMap<String,Class> StrToClass = new HashMap<String,Class>();
	/** Mapping from class to type identifier */
	private HashMap<Class,String> ClassToStr = new HashMap<Class,String>();

	/** Add a new mapping to the lists 
	 * @param ident Type identifier
	 * @param c Java class 
	 */
	public void put (String ident, Class c)
	{
		StrToClass.put(ident, c);
		ClassToStr.put(c, ident);
	}

	/** Get the type identifier for a class
	 * @param c Java class
	 * @return Type identifier
	 */
	public String get (Class c)
	{
		String ret = ClassToStr.get(c);
		if (ret == null)
			ret = new ClassInfo(c).fqcncs();
		return ret;
	}

	/** Get the class for a type identifier
	 * @param ident Type identifier
	 * @return Java class 
	 */
	public Class get (String ident) throws ClassNotFoundException
	{
		Class ret = StrToClass.get(ident);
		if (ret == null)
		{
			ident = ClassInfo.cncs2java(ident);
			ret = Refl.getClass(ident);
			if (ret == null)
				ret = Class.forName(ident);
		}
		return ret;
	}

	/** Retrieve a string representation of this object
	 * @return String
	 */
	public String toString ()
	{
		StringBuffer ret = new StringBuffer();
		for (Class c : ClassToStr.keySet())
			ret.append(c.getName()+" -> "+ClassToStr.get(c));
		return ret.toString();
	}
}
