package CTL.Comm;

import CTL.Types.*;

import java.io.*;
import java.net.*;

/** Base class for all CTL communicators */
public abstract class Communicator
{	
	/** Flag for signalling */
	@SuppressWarnings("unused")
	private boolean running = false;
	
	/** Notifies this communicator of a critical error */
	public final void notifyErr ()
	{
		running = false;
		System.exit(10);
	}
	
	public abstract PeerID pid ();

	public abstract byte[] recv () throws CTLException, IOException;
	public abstract byte[] recv (long len) throws CTLException, IOException;

	public abstract void send (byte[] data, long len, boolean debug) throws IOException;

	public final void send (byte[] data, boolean debug) throws IOException
	{
		send(data, data.length, debug);
	}

	public final void send (byte[] data, long len) throws IOException
	{
		send(data, data.length, true);
	}

	public final void send (byte[] data) throws IOException
	{
		send(data, data.length);
	}

	public abstract void accept () throws IOException;
	public abstract void connect (PeerID remote) throws IOException, UnknownHostException;
	public abstract void listen () throws IOException;
	public abstract void close () throws IOException;
}
