package CTL.Streams;

import java.io.*;
import java.nio.*;

/** Replacement for Java's DataInputStream */
public class DataInputStream2 implements DataInput
{
	private DataInputStream stream;
	private boolean swapBytes = false;

	public DataInputStream2 (InputStream stream)
	{
		this.stream = new DataInputStream(stream);
	}

	public void changeEndianess (ByteOrder order)
	{
		if (order == ByteOrder.LITTLE_ENDIAN)
			swapBytes = true;
		else
			swapBytes = false;
	}

	public boolean readBoolean () throws IOException
	{
		return stream.readBoolean();
	}

	public byte readByte () throws IOException
	{
		return stream.readByte();
	}

	public char readChar () throws IOException
	{
		char read = stream.readChar();
		if (swapBytes)
			read = Character.reverseBytes(read);
		return read;
	}

	public double readDouble () throws IOException
	{
		double read = stream.readDouble();
		if (swapBytes)
			read = LittleEndian.doubleReverseBytes(read);
		return read;
	}

	public float readFloat () throws IOException
	{
		float read = stream.readFloat();
		if (swapBytes)
			read = LittleEndian.floatReverseBytes(read);
		return read;
	}

	public void readFully (byte[] b) throws IOException
	{
		stream.readFully(b);
	}

	public void readFully (byte[] b, int off, int len) throws IOException
	{
		stream.readFully(b, off, len);
	}

	public int readInt () throws IOException
	{
		int read = stream.readInt();
		if (swapBytes)
			read = Integer.reverseBytes(read);
		return read;
	}

	@SuppressWarnings("deprecation")
	public String readLine () throws IOException
	{
		return stream.readLine();
	}

	public long readLong () throws IOException
	{
		long read = stream.readLong();
		if (swapBytes)
			read = Long.reverseBytes(read);
		return read;
	}

	public short readShort () throws IOException
	{
		short read = stream.readShort();
		if (swapBytes)
		{
			CTL.Env.log.msg(CTL.Logger.DBG2, "swapping");
			read = Short.reverseBytes(read);
		}
		return read;
	}

	public int readUnsignedByte () throws IOException
	{
		int read = stream.readUnsignedByte();
		if (swapBytes)
			read = Integer.reverseBytes(read);
		return read;
	}

	public int readUnsignedShort () throws IOException
	{
		int read = stream.readUnsignedShort();
		if (swapBytes)
			read = Integer.reverseBytes(read);
		return read;
	}

	public String readUTF () throws IOException
	{ 
		return stream.readUTF();
	}

	public int skipBytes (int n) throws IOException
	{
		return stream.skipBytes(n);
	}

	public int available () throws IOException
	{
		return stream.available();
	}

	public int read (byte[] b) throws IOException
	{
		return stream.read(b);
	}
}
