package CTL.Types;

import CTL.*;
import CTL.Serialize.*;

import java.io.*;
import java.lang.reflect.*;

/** Group information storage */
public class GroupInfo implements Writable
{
	/** Local Peer ID */
	private PeerID pid;
	/** ObjectID a Group object */
	private long objID;

	private boolean wantTerm = true;
	
	/** Create a new GroupInfo object
	 * @param pid PeerID
	 * @param objID ObjectID
	 */
	public GroupInfo (PeerID pid, long objID)
	{
		this.pid = pid;
		this.objID = objID;
		if (pid != null) // Heh, w/o this, logging causes a bug...
			Env.log.msg(Logger.DBG2, "GroupInfo port set to "+pid.port()+".");
	}

	/** Retrieve a string representation of this object
	 * @return String
	 */
	public String toString ()
	{
		return pid.toString();
	}

	/** Serial read function
	 * @param in Input stream
	 */
	public void read (SerialIn in) throws IOException, ClassNotFoundException, 
		InstantiationException, IllegalAccessException, InvocationTargetException
	{
		pid = in.serialRead(PeerID.class);
		objID = in.serialRead(Long.class);
		wantTerm = in.serialRead(Boolean.class);
	}
	
	/** Serial write function
	 * @param out Output stream
	 */
	public void write (SerialOut out) throws IOException, IllegalAccessException,
		InvocationTargetException, CTLException
	{
		out.serialWrite(pid);
		out.serialWrite(objID);
		out.serialWrite(wantTerm);
	}

	/** Retrieve the IP address
	 * @return IPaddr
	 */
	public IPaddr host ()
	{
		return pid.host();
	}

	/** Retrieve the port
	 * @return Port
	 */
	public int port ()
	{
		return pid.port();
	}

	/** Retrieve the PeerID
	 * @return PeerID
	 */
	public PeerID pid ()
	{
		return pid;
	}

	/** Retrieve the objID
	 * @return ObjectID
	 */
	public long id ()
	{
		return objID;
	}
}
