package CTL.Types;

import CTL.Serialize.*;
import java.io.*;
import java.lang.reflect.*;
import java.net.*;

/** ID of a remote peer */
public class PeerID implements Writable
{
	/** IP address */
	private IPaddr host;
	/** Port */
	private int port;

	/** Generate a new PeerID
	 * @param host Host
	 * @param port Port
	 */
	public PeerID (IPaddr host, int port)
	{
		this.host = host;
		this.port = port;
	}

	public PeerID (String host, int port) throws UnknownHostException
	{
		this(new IPaddr(host), port);
	}

	/** Retrieve a string representation of this object
	 * @return String
	 */
	public String toString ()
	{
		return host+":"+port;
	}

	/** Retrieve the IP address of the peer
	 * @return IP address 
	 */
	public IPaddr host ()
	{
		return host;
	}

	/** Retrieve the port of the peer
	 * @return Port
	 */
	public int port ()
	{
		return port;
	}

	/** Set the IP address
	 * @param host IP address
	 */
	public void setHost (IPaddr host)
	{
		this.host = host;
	}

	/** Set the port
	 * @param port Port
	 */
	public void setPort (int port)
	{
		this.port = port;
	}

	/** Serial read function
	 * @param in Input stream
	 */
	public void read (SerialIn in) throws IOException, ClassNotFoundException, 
		InstantiationException, IllegalAccessException, InvocationTargetException
	{
		//System.out.println("DBG :: PeerID.read() :: begin");
		port = in.readInt();
		//System.out.println("DBG :: PeerID.read() :: "+port);
		host = in.serialRead(IPaddr.class);
		//System.out.println("DBG :: PeerID.read() :: "+host);
		//System.out.println("DBG :: PeerID.read() :: end");
	}
	
	/** Serial read function
	 * @param out Output stream
	 */
	public void write (SerialOut out) throws IOException, IllegalAccessException,
		InvocationTargetException, CTLException
	{
		out.writeInt(port);
		out.serialWrite(host);
	}

	/** Determine the size of this type
	 * @return Size (in bytes)
	 */
	public int size ()
	{
		return 2*8+4;
	}
}
