/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */

package CodeGen;

import CTL.Annotate.*;
import ReflWrap.*;

import java.io.*;
import java.lang.reflect.*;

/**
 * Generates the NameDebug components used by the Debugger
 * @author Johannes Baumgarten,Jurij Merker,Bjoern Gebhardt,Stephan Kluge,Svetlana Li
 */
public class GenDebug extends Gen
{
	public GenDebug (String klass) throws ClassNotFoundException
	{
		super(klass);
		name = classNamer(cinfo, "Debug");
		supername = classNamer(cinfo, "CI");
	}
	
	protected void gen (OutputStreamWriter ostr) throws IOException, NoSuchMethodException
	{
		writeTo(ostr, genHead());
		writeTo(ostr, "private "+name+" self = null;\n");
		// Dummy constructor
		writeTo(ostr, "public "+name+ " ()\n{\nsuper((Object)null);\n}\n");
		writeTo(ostr, genAccept());
		writeTo(ostr, "}\n");
	}

	protected String genAccept () throws IOException
	{
		return genAccept(this);
	}
	
	
	 protected String genHead () throws IOException
	    {
	        StringBuffer ret = new StringBuffer();

	        ret.append("package "+pkg+";\n\n");
	        ret.append("import CTL.*;\n");
	        ret.append("import CTL.CCompat.*;\n");
	        ret.append("import CTL.Comm.*;\n");
	        ret.append("import CTL.Streams.*;\n");
	        ret.append("import CTL.Types.*;\n");
	        ret.append("import ReflWrap.*;\n\n");
	        ret.append("import java.io.*;\n");
	        ret.append("import java.net.*;\n");
	        ret.append("import java.lang.reflect.*;\n");
			ret.append("import java.util.*;\n");
			ret.append("import Debug.*;\n");
	        if (cinfo._package().equals(""))
	        {
	            System.err.println("Implementation belongs to no package.");
	            System.exit(1);
	        }
	        else
	            ret.append("import "+cinfo._package()+".*;\n");
	        getImports(ret, cinfo.fqcn());

	        ret.append("\n/* This class was autogenerated by the CTL4j code"+
	            " generator from\n *\t\t"+getFQCN()+".\n * Written by Boris"+
	            " Buegling, licensed under the GNU General Public license. */\n");
	        
			ret.append("public "+modifiers+" class "+name+" extends "+supername+"\n{\n");
	        return ret.toString();
	    }
	
	protected  String genAccept(Gen self) throws IOException
	{	
		StringBuffer ret = new StringBuffer();
		StringBuffer ret2 = new StringBuffer();
		int j, st=1,me=1;
		boolean _const = self.cinfo.isAnot(const_.class);

		ret.append("@SuppressWarnings(\"unchecked\")\n");
		ret.append("synchronized public static void accept (OIStream in, Header ohead, long objID, FID fid, int id, Env env) throws IOException, ClassNotFoundException, InstantiationException, IllegalAccessException, NoSuchMethodException, InvocationTargetException, CTLException\n");
		ret.append("{\nClass[] args2 = null;\n");
		ret.append("Header head = null;\n");
		ret.append("long ID = 0;\n");
		ret.append("int tag = 0;\n");
		ret.append("IStream2 args = null;\n");
		ret.append("rPointer rp = null;\n");
		ret.append("OOStream out = null;\n");
		ret.append("String arguments = null;\n");
		ret.append("IPaddr host = ohead.pid().host();\n");
		ret.append("String rvalues = null;\n");
		ret.append("int port = ohead.pid().port();\n");
		ret.append("Env.log.msg(Logger.DBG2, ohead+\" \"+objID);\n");
		ret.append("if (objID==0)\n{\n");
		ret.append("char c = fid.name().charAt(fid.name().length()-1);\n");
		ret.append("Env.log.msg(Logger.DBG2, new Integer(c).toString());\n");
		ret.append("Env.log.msg(Logger.DBG2, fid.toString());\n");	
		
		ret.append("switch (c)\n{\n");
		ret.append("case 'C':\n");
		ret.append("switch (fid.ID())\n{\n");

		// Constructors
		for (int i=0;i<self.cinfo.constructors().length;i++)
		{
			Constructor cnst = self.cinfo.constructors()[i];
			if (cnst.isAnnotationPresent(cdont.class))
				continue;
			ConstructInfo cinfo2 = new ConstructInfo(cnst);
			
			ret.append("case "+i+":\n");
			if (cinfo2.params().length<1){
				ret.append("Object[] foo"+i+" = null;\n");
			
			}
			else
			{
				ret.append("Object[] foo"+i+" = new Object["+
					cinfo2.params().length+"];\n");
				
				for (j=0;j<cinfo2.params().length;j++)
					genArgList(ret, cinfo2.params()[j], i, j, true,	cinfo2.getParamAnot(j));	
			}
			ret.append("rp = in.serialRead(rPointer.class);\n");
			ret.append("Env.log.msg(Logger.DBG, \"\"+rp);\n");
			ret.append("tag = in.readInt();\n");
			ret.append("Class moongooc"+i+" = Class.forName(\"ReflWrap.TypeTree\");\n");
			ret.append("Constructor cc"+i+" = rResult.class.getConstructor(moongooc"+i+");\n");
			ret.append("Object[] objsc"+i+" = new Object[1];\n");
			ret.append("objsc"+i+"[0] = (Object)new TypeTree(rPointer.class);\n");
			ret.append("Object rpo"+i+" = cc"+i+".newInstance(objsc"+i+"[0]);\n");
			ret.append("Env.map.regObj(rpo"+i+",(int)rp.objID());\n");
			
			ret.append("EnvDebug.mirror.setConstrList(EnvDebug.mirror.getPartner(ohead.pid()),tag,fid._class());\n");
			ret.append("EnvDebug.mirror.setMTagList(ohead.pid(),tag,false);\n");
			
			ret.append("if (!EnvDebug.mirror.getComponent(EnvDebug.mirror.getPartner(ohead.pid()).host(),fid._class()))\n"); 
			ret.append("{\n");
			ret.append("EnvDebug.mirror.setCompoList(EnvDebug.mirror.getPartner(ohead.pid()).host(),fid._class());\n");
			ret.append("EnvDebug.vis.addComponent(EnvDebug.mirror.getPartner(ohead.pid()).toString(),fid._class());\n");
			ret.append("}\n");

			if (cinfo2.params().length>0)
			{
				ret.append("arguments = \"\"+");
				for (j=0;j<cinfo2.params().length-1;j++)
					ret.append("foo"+i+"["+j+"]+\", \"+");
				ret.append("foo"+i+"["+(cinfo2.params().length-1)+"];\n");
			}
			else
				ret.append("arguments = \"\";\n");
			
			ret.append("EnvDebug.vis.addMessage(ohead.pid().toString(), null,"+
			         "0,\"\"+tag, fid.name(), rvalues,"+
			         "arguments, EnvDebug.mirror.getPartner(ohead.pid()).toString(), fid._class(),"+
			         "0);\n");
			ret.append("if (EnvDebug.mirror.runmode_enabled)\n");
			ret.append("{\n");
			ret.append("	EnvDebug.mirror.sendContinuous(ohead.pid());\n");
			ret.append("	EnvDebug.vis.removeMessage(ohead.pid().toString(),null,0,\"\"+EnvDebug.mirror.getMTag(ohead.pid()),EnvDebug.mirror.getPartner(ohead.pid()).toString(),null,0);");
			ret.append("	EnvDebug.mirror.removePeerIDElement(ohead.pid());\n");
			ret.append("}\n");
			ret.append("else\n");
			ret.append("	EnvDebug.mirror.setContinuousList(ohead.pid(),fid,fid._class(),false);\n");
			ret.append("break;\n");
		}	
	
		ret.append("}\nbreak;\ncase 'S':\n");
		ret.append("switch (fid.ID())\n{\n");
		for (int i=0;i<self.cinfo.methods().length;i++)
		{
			Method moo = self.cinfo.methods()[i];
			Methode meth = methods[i];
			// If there are unassigned FIDs, skip them.
			if (moo == null)
				continue;
			MethodInfo minfo = new MethodInfo(moo);
			
			if (moo.isAnnotationPresent(dont.class))
			{
				if (minfo.isStatic())
					st++;
				else
					me++;
				
				continue;
			}
			
			if (minfo._class()!=self.cinfo._class())
				continue;
				
			StringBuffer loc;
			if (minfo.isStatic())
			{
				loc = ret;
				loc.append("case "+st+":\n");
				st++;
			}
			else
			{
				loc = ret2;
				loc.append("case "+me+":\n");
				me++;
			}
			
			for (j=0;j<minfo.params().length;j++)
				genArgList(loc, minfo.params()[j], i, j, false, 
					minfo.getParamAnot(j));

			if (!moo.isAnnotationPresent(builtin.class))
				loc.append("rp = in.serialRead(rPointer.class);\n");
			loc.append("tag = in.readInt();\n");
			
			loc.append("Class moongoo"+i+" = (new TypeTree[0]).getClass();\n");
			loc.append("Constructor c"+i+" = "+
					"rResult.class.getConstructor(moongoo"+i+");\n");
			
			// Code for the rResult construction
			int nret=0;
			for (int k=0;k<meth.minfo().getParams().length;k++) 
				if (!Refl.hasAnot("@CTL.Annotate.const_()", 
							meth.minfo().getParamAnot(k)) && !meth._const())
					nret++;
			loc.append("TypeTree[] argz"+i+" = new TypeTree["+(nret+1)+"];\n");
			int p=0;
			for (int k=0;k<meth.minfo().getParams().length;k++) 
				if (!Refl.hasAnot("@CTL.Annotate.const_()", 
						meth.minfo().getParamAnot(k)) && !meth._const())
				{
					String arg = Refl.class_(meth.minfo().getParams()[k]); 
					if (ByteCode.matches(meth.minfo().params()[k], ".*<.*"))
						arg = "\""+meth.minfo().params()[k]+"\"";
					loc.append("argz"+i+"["+(p++)+"] = new TypeTree("+
						arg+");\n");
				}

			String arg = Refl.class_(meth.minfo().returns()._class()); 
			if (ByteCode.matches(meth.returnsT(), ".*<.*"))
				arg = "\""+meth.returnsT()+"\""; 
			loc.append("argz"+i+"["+nret+"] = new TypeTree("+arg+");\n"+
				"Object[] objs"+i+" = new Object[1];\n"+
				"objs"+i+"[0] = (Object)argz"+i+";\n"+ 
				"Object obj"+i+" = c"+i+".newInstance(objs"+i+"[0]);\n"+
				"Env.map.regObj(obj"+i+",(int)rp.objID());\n");
				
			if (minfo.returnsT().equals("void")) 
			{
				loc.append("rvalues =\"void\"  ;\n");
				loc.append("EnvDebug.mirror.setMTagList(ohead.pid(),tag,true);\n");
			}
			else
			{
				loc.append("EnvDebug.mirror.setMTagList(ohead.pid(),tag,false);\n");
			}
					
			if (minfo.params().length>0)
			{
				loc.append("arguments = \"\"+");
				for (j=0;j<minfo.params().length-1;j++)
					loc.append("arg"+j+i+"+\", \"+");
				loc.append("arg"+(minfo.params().length-1)+i+";\n");
			}
			else
				loc.append("arguments = \"\";\n");
			
			loc.append("if (objID>0)\n");
			loc.append("{\n");			
			loc.append("EnvDebug.vis.addMessage(ohead.pid().toString(),null,"+
			         "0,\"\"+tag, fid.name(), rvalues,"+
			         "arguments, EnvDebug.mirror.getPartner(ohead.pid()).toString(),EnvDebug.mirror.getClassName(objID),"+
			         "objID);\n");
			loc.append("}\n");
			loc.append("else\n");	
			loc.append("{\n");			
			loc.append("if (!EnvDebug.mirror.getComponent(EnvDebug.mirror.getPartner(ohead.pid()).host(),fid._class()))\n"); 
			loc.append("{\n");
			loc.append("EnvDebug.mirror.setCompoList(EnvDebug.mirror.getPartner(ohead.pid()).host(),fid._class());\n");
			loc.append("EnvDebug.vis.addComponent(EnvDebug.mirror.getPartner(ohead.pid()).toString(),fid._class());\n");
			loc.append("}\n");			
			loc.append("EnvDebug.vis.addMessage(ohead.pid().toString(),null,"+
			         "0,\"\"+tag, fid.name(), rvalues,"+
			         "arguments,EnvDebug.mirror.getPartner(ohead.pid()).toString(),null,"+
			         "0);\n");
			loc.append("}\n");
			loc.append("\n");
			
			loc.append("if (EnvDebug.mirror.more)\n");
			loc.append("	System.out.println(\"arguments :\"+arguments);\n");
			loc.append("\n");
			
			loc.append("if (EnvDebug.mirror.runmode_enabled)\n");
			loc.append("{\n");
			loc.append("	EnvDebug.mirror.sendContinuous(ohead.pid());\n");
			loc.append("	EnvDebug.vis.removeMessage(ohead.pid().toString(),null,0,\"\"+EnvDebug.mirror.getMTag(ohead.pid()),EnvDebug.mirror.getPartner(ohead.pid()).toString(),null,0);");
			loc.append("	EnvDebug.mirror.removePeerIDElement(ohead.pid());\n");
			loc.append("}\n");
			loc.append("else\n");
			loc.append("	EnvDebug.mirror.setContinuousList(ohead.pid(),fid,fid._class(),false);\n");
						
			
			loc.append("break;\n");
		}
		ret2.append("default: \n ");
		ret2.append("System.out.println(\"Default Block erreicht!\");\n");
		ret2.append("if (EnvDebug.mirror.runmode_enabled)\n");
		ret2.append("{\n");
		ret2.append("	EnvDebug.mirror.sendContinuous(ohead.pid());\n");
		ret2.append("	EnvDebug.vis.removeMessage(ohead.pid().toString(),null,0,\"\"+EnvDebug.mirror.getMTag(ohead.pid()),EnvDebug.mirror.getPartner(ohead.pid()).toString(),null,0);");
		ret2.append("	EnvDebug.mirror.removePeerIDElement(ohead.pid());\n");
		ret2.append("}\n");
		ret2.append("else\n");
		ret2.append("	EnvDebug.mirror.setContinuousList(ohead.pid(),fid,fid._class(),false);\n");
		ret2.append("break;\n");
		
		ret.append("default: \n ");
		ret.append("System.out.println(\"Default Block erreicht!\");\n");
		ret.append("if (EnvDebug.mirror.runmode_enabled)\n");
		ret.append("{\n");
		ret.append("	EnvDebug.mirror.sendContinuous(ohead.pid());\n");
		ret.append("	EnvDebug.vis.removeMessage(ohead.pid().toString(),null,0,\"\"+EnvDebug.mirror.getMTag(ohead.pid()),EnvDebug.mirror.getPartner(ohead.pid()).toString(),null,0);");
		ret.append("	EnvDebug.mirror.removePeerIDElement(ohead.pid());\n");
		ret.append("}\n");
		ret.append("else\n");
		ret.append("	EnvDebug.mirror.setContinuousList(ohead.pid(),fid,fid._class(),false);\n");
		ret.append("break;\n");
		ret.append("}\nbreak;\n}\n");
		ret.append("}\nelse\n{\n"); 
		ret.append("Env.log.msg(Logger.DBG2, fid.toString());\n");
		
		ret.append("if (objID == EnvDebug.mirror.getEOCObjID(EnvDebug.mirror.getPartner(ohead.pid()))& fid.ID()==3)\n");
		ret.append("{\n");
		ret.append("if (EnvDebug.mirror.runmode_enabled)\n");
		ret.append("{\n");
		ret.append("	EnvDebug.mirror.sendContinuous(ohead.pid());\n");
		ret.append("	EnvDebug.vis.removeHost(EnvDebug.mirror.getPartner(ohead.pid()).toString());\n");
		ret.append("	EnvDebug.vis.removeHost(ohead.pid().toString());");
		ret.append("	EnvDebug.mirror.removePeerID(EnvDebug.mirror.getPartner(ohead.pid()));\n");
		ret.append("	EnvDebug.mirror.removePeerID(ohead.pid());\n");
		ret.append("	EnvDebug.mirror.removeEOC(ohead.pid());\n");
		ret.append("}\n");
		ret.append("else\n");
		ret.append("{\n");
		ret.append("	EnvDebug.mirror.setContinuousList(ohead.pid(),fid,fid._class(),true);\n");
		ret.append("}\n");
		
		ret.append("return;");
		ret.append("}\n");
		
		ret.append("switch (fid.ID())\n{\n"); 
		ret.append(__putRefCount());
		ret.append(ret2.toString());
		ret.append("}\n}\n");
		ret.append("Env.log.msg(Logger.DBG2, \"Message Tag: \"+tag);\n");
		ret.append("}\n");

		return ret.toString();
	}
}		
