package Example;

import CTL.*;
import CTL.Types.*;
import javaSys.*;

import java.util.*;

/** 
 * This client demonstrates parallel execution of methods using multiple
 * processes, which can live on different physical machines, of course.
 */
public class Client13
{
	public static void main (String[] args)
	{
		CTL.Timer all = new CTL.Timer();
		LinkedList<Location> locs = Location.parseFile("locs.txt");
		if (locs.size() < 2)
			throw new RuntimeException("Example.Client13 needs at least 2 valid locations.");
		// TODO: Spawning of processes only works sequentially
		// TODO: Using locs.get($i) sucks, roundtrip location usage should be
		// embedded into the default process allocator
		CTL.Process proc1 = new CTL.Process(locs.get(0));
		CTL.Process proc2 = new CTL.Process(locs.get(1));

		String procs = CTL.Timer.timestr(all.stop2());

		CTestCI.use(proc1);
		CTestCI test1 = new CTestCI();
		EvCalcCI.use(proc2);
		EvCalcCI test2 = new EvCalcCI();

		CTL.Timer t = new CTL.Timer();

		// TODO: This sucks, too, calling the processes takes longer than the
		// calls themselves take, *_rr() should return immediately
		// These two methods are called in parallel
		rResult rr1 = test1.add_rr(3, 4);
		rResult rr2 = test2.barfoo_rr(4, 7);

		String calls = CTL.Timer.timestr(t.stop2());

		// Waiting for the results...
		while (!(rr1.received() && rr2.received()));
		t.stop();
		all.stop();

		System.out.println("Elapsed time (in seconds): "+all+"/"+procs+"/"+t+"/"+calls);
		System.out.println("test1.add(3, 4) = "+rr1.value());
		System.out.println("test2.barfoo(4, 7) = "+rr2.value());
		System.out.println("Object 1 lives on "+test1.peerID());
		System.out.println("Object 2 lives on "+test2.peerID());
	}
}
