package Hitman;

import java.io.*;
import java.util.*;

/** IO utility functions */
public class IOUtil
{
	/** Read a string from a Reader.
	 * @param in Reader to read from
	 * @param len Number of bytes to read
	 * @return The String
	 */
	public static String slurp (Reader in, long len) throws IOException
	{
		if (len<1)
			return "";

		StringBuffer buf = new StringBuffer();

		for (long i=0;i<len;i++)
			buf.append((char)in.read());

		return buf.toString();
	}

	/** Read a String from a Reader until EOF
	 * @param in Reader to read from
	 * @return The String
	 */
	public static String slurp (Reader in) throws IOException
	{
		StringBuffer buf = new StringBuffer();
		int c;
		while ((c = in.read()) != -1)
			buf.append((char)c);
		return buf.toString();
	}

	// Untested.
	public static String slurp (Reader r, byte __unused) throws IOException
	{
		BufferedReader in;
		if (r instanceof BufferedReader)
			in = (BufferedReader)r;
		else
			in = new BufferedReader(r);
		
		boolean read = true;
		while (read)
		{
			String size = in.readLine();
			int sz = Integer.parseInt(size);
			System.out.println(sz);
			if (sz==0)
				read = false;
			else
			{
				char[] buf = new char[sz];
				in.read(buf, 0, sz);
			}
		}

		return "";
	}
	
	/** Execute a command (system() equivalent)
	 * @param cmd The command to execute
	 * @return Everything that was written to stdout and stderr by the
	 * 	executed process
	 */
	public static String exec (String cmd) throws IOException
	{
		Runtime rt = Runtime.getRuntime();
		Process proc = rt.exec(cmd);
		
		InputStream in = proc.getInputStream();
		InputStreamReader read = new InputStreamReader(in);
		BufferedReader rd = new BufferedReader(read);
		InputStream in2 = proc.getErrorStream();
		InputStreamReader read2 = new InputStreamReader(in2);
		BufferedReader rd2 = new BufferedReader(read2);
		
		try
		{
			proc.waitFor();
		}
		catch (Exception e) 
		{
			e.printStackTrace();
		}

		StringBuffer buf = new StringBuffer();
		while (rd.ready())
			buf.append(rd.readLine()+"\n");
		while (rd2.ready())
			buf.append(rd2.readLine()+"\n");
		
		return buf.toString();
	}
	
	/** Determine the MIMEtype of a file (UNIX-specific function).
	 * @param file The relevant file
	 * @return MIMEtype as returned by file(1)
	 */
	public static String getFileType (File file) throws IOException
	{
		String cmd = "file "+file.getAbsolutePath();
		String outp = exec(cmd);

		StringTokenizer tok = new StringTokenizer(outp, ":");
		String moo = tok.nextToken();
		String type = outp.substring(moo.length()+2, outp.length()-1);
		
		if (type.equals("ASCII English text") ||
			type.equals("Bourne shell script text executable"))
			return "text/plain";

		System.out.println("Unrecognized type '"+type+"'.");
		return "application/octet-stream";
	}
	
	/** Split the extension from a filename.
	 * @param file The relevant file
	 * @return The file's extension
	 */
	public static String getExt (File file)
	{
		String fname = file.toString();
		if (fname.length()<4)
			return "";
		String ext = fname.substring(fname.length()-4, fname.length());
		if (ext.charAt(0)!='.')
			return "";
		return ext.substring(1, ext.length());
	}
}
