package NCLib;

import java.io.*;
import java.net.*;
import java.util.*;
import java.util.jar.*;

public class Classes
{
	/** Blacklist of classes with unwanted code in their static-Block */
	private static LinkedList<String> blacklist;

	static
	{
		blacklist = new LinkedList<String>();
		blacklist.add("Debug.EnvDebug");
		blacklist.add("Test.TClient");
	}

	public static LinkedList<Class> getClasses ()
	{
		LinkedList<Class> res = new LinkedList<Class>();
		for (String pkg : getPackages())
		{
			LinkedList<Class> list = getClasses(pkg);
			if (list != null)
				res.addAll(list);
		}
		for (String pkg : getPackagesFromClasspath())
		{
			LinkedList<Class> list = getClasses(pkg);
			if (list != null)
				res.addAll(list);
		}
		return res;
	}

	// TODO: System-Resource URL is null for built-in packages
	public static LinkedList<Class> getClasses (String pkg)
	{
		LinkedList<Class> res = new LinkedList<Class>();
		URL url = ClassLoader.getSystemClassLoader().getSystemResource(pkg.replace(".", "/"));
		//System.out.println("DEBUG :: "+pkg+" :: "+url);
		if (url == null)
			return null;
		File dir = new File(url.getFile());
		if (dir.exists())
		{
			for (String fname : dir.list())
			{
				if (!fname.endsWith(".class"))
				{
					if (new File(dir+"/"+fname).isDirectory())
					{
						LinkedList<Class> list = getClasses(pkg+"."+fname);
						if (list != null)
							res.addAll(list);
						continue;
					}

					if (fname.endsWith(".jar"))
					{
						for (String pkg2 : getDirectoriesFromJAR(dir+"/"+fname))
						{
							LinkedList<Class> list = getClasses(pkg2);
							if (list != null)
								res.addAll(list);
						}
						continue;
					}

					//System.out.println("DEBUG :: "+fname);
				}

				try
				{
					String name = pkg+"."+fname.replace(".class", "");
					if (blacklist.indexOf(name) > -1)
						continue;
					//System.out.println(name);
					Class cl = Class.forName(name);
					res.add(cl);
				}
				catch (ClassNotFoundException e)
				{
					continue;
				}
			}
		}
		else
		{
			if (url.getProtocol().equals("jar"))
			{
				String pkgAsFileName = pkg.replace(".", "/");
				String fname = dir.toString().replace("file:", "");
				fname = fname.substring(0, fname.indexOf("!"));
				try
				{
					JarFile jar = new JarFile(fname);
					Enumeration<JarEntry> entries = jar.entries();
					while (entries.hasMoreElements())
					{
						String entry = entries.nextElement().toString();
						if (!entry.startsWith(pkgAsFileName))
							continue;
						if (entry.equals(pkgAsFileName+"/"))
							continue;
						if (entry.split("/").length > 1+pkgAsFileName.split("/").length)
							continue;
						if (entry.endsWith("/"))
						{
							String basename = new File(entry).getName();
							LinkedList<Class> list = getClasses(pkg+"."+basename);
							if (list != null)
								res.addAll(list);
							continue;
						}
						if (entry.endsWith(".class"))
						{
							try
							{
								String name = entry.replace("/", ".").replace(".class", "");
								Class cl = Class.forName(name);
								res.add(cl);
							}
							catch (ClassNotFoundException e) 
							{
								//e.printStackTrace();
								continue;
							}
							catch (NoClassDefFoundError e)
							{
								continue;
							}
						}
						else
						{
							//System.out.println("DEBUG :: "+fname);
						}
					}
				}
				catch (IOException e) 
				{
					return res;
				}
			}
		}
		return res;
	}

	private static LinkedList<String> getPackagesFromClasspath ()
	{
		LinkedList<String> res = new LinkedList<String>();
		for (String element : System.getenv("CLASSPATH").split(":"))
		{
			if (element.endsWith(".jar"))
			{
				for (String fname : getDirectoriesFromJAR(element))
					res.add(fname.substring(0, fname.length()-1));
			}
			else
			{
				File dir = new File(element);
				if (dir.exists())
				{
					for (String fname : dir.list())
						if (new File(dir+"/"+fname).isDirectory())
							res.add(fname);
				}
			}
		}
		return res;
	}

	private static LinkedList<String> getDirectoriesFromJAR (String fname)
	{
		LinkedList<String> res = new LinkedList<String>();
		try
		{
			JarFile jar = new JarFile(fname);
			Enumeration<JarEntry> entries = jar.entries();
			while (entries.hasMoreElements())
			{
				String entry = entries.nextElement().toString();
				if (entry.endsWith("/") && (entry.split("/").length < 2))
					res.add(entry);
			}
		}
		catch (IOException e) 
		{
			//e.printStackTrace();
		}
		return res;
	}

	private static LinkedList<String> getPackages ()
	{
		LinkedList<String> res = new LinkedList<String>();
		for (Package pkg : Package.getPackages())
			res.add(pkg.getName());
		return res;
	}

	public static void main (String[] args)
	{
		System.out.println(getClasses().size());
	}
}
