package NCLib;

import java.io.*;
import java.util.*;

/** Various helper functions concerning files */
public class Files
{
	/** Read a whole file in at once 
	 * @param fname Filename
	 * @return The file's contents
	 */
	public static String readAll (String fname)
	{
		StringBuffer ret = new StringBuffer();
		BufferedReader inp = null;
		
		try
		{
			FileReader reader = new FileReader(fname);
			inp = new BufferedReader(reader);
			
			String buf = "";
			do
			{
				if (!buf.equals(""))
					ret.append(buf+"\n");
				buf = inp.readLine();
			} while (buf != null);
		} 
		catch (Exception e) {}
		finally
		{
			if (inp != null)
			{
				try
				{
					inp.close();
				}
				catch (Exception e) {}
			}
		}
		return ret.toString();
	}

	/** Write a whole file at once
	 * @param fname Filename of the file to write
	 * @param content What to write to the file
	 */
	public static void writeAll (String fname, String content)
	{
		BufferedWriter out = null;
		
		try
		{
			FileWriter writer = new FileWriter(fname);
			out = new BufferedWriter(writer);
			out.write(content);
		}
		catch (Exception e) {}
		finally
		{
			if (out != null)
			{
				try
				{
					out.close();
				}
				catch (Exception e) {}
			}
		}
	}

	/** Attempt to create a directory and exit the program if it isn't possible
	 * @param dirname Name of the directory
	 */
	public static void mkdirOrDie (String dirname)
	{
		if (!new File(dirname).mkdir())
		{
			System.out.println(dirname+": Could not create directory.");
			System.exit(20);
		}
	}

	/** Create a symbolic link (Unix-only, needs ln(1))
	 * @param src Source filename
	 * @param dest Destination filename
	 */
	public static void symlink (String src, String dest)
	{
		try
		{
			Process.exec("ln -s "+src+" "+dest);
		}
		catch (Exception e) {}
	}

	/** Change the permissions of a file (Unix-only, needs chmod(1))
	 * @param perms Permissions
	 * @param fname Filename
	 */
	public static void changePermissions (String perms, String fname)
	{
		try
		{
			Process.exec("chmod "+perms+" "+fname);
		}
		catch (Exception e) {}
	}

	/** Test routine */
	public static void main (String[] args)
	{
		String in = readAll("data/test.tmpl");
		writeAll("/tmp/test", in);
	}
}
